@@ -44,7 +44,7 @@ void skin_repaint_annunciator(cairo_t *cr, int which);
 void skin_invalidate_annunciator(GdkWindow *win, int which);
 void skin_find_key(int x, int y, bool cshift, int *key, int *code);
 int skin_find_skey(int ckey);
-unsigned char *skin_find_macro(int ckey, bool *is_name);
+unsigned char *skin_find_macro(int ckey, int *type);
 unsigned char *skin_keymap_lookup(guint keyval, bool printable,
                                   bool ctrl, bool alt, bool shift, bool cshift,
                                   bool *exact);
@@ -56,5 +56,8 @@ bool need_to_paint_only_display(cairo_t *cr);
 void skin_repaint_display(cairo_t *cr);
 void skin_invalidate_display(GdkWindow *win);
 void skin_display_set_enabled(bool enable);
+void skin_get_size(int *width, int *height);
+void skin_set_window_size(int width, int height);
+void skin_get_window_size(int *width, int *height);
 
 #endif
